@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2005-2006 by Olivier Goffart <ogoffart at kde.org>
+   Copyright (C) 2005-2009 by Olivier Goffart <ogoffart at kde.org>
    Copyright (C) 2008 by Dmitry Suzdalev <dimsuz@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,8 @@
 #include <khbox.h>
 #include <kvbox.h>
 
+#include <QBuffer>
+#include <QImage>
 #include <QLabel>
 #include <QTextDocument>
 #include <QApplication>
@@ -36,9 +38,9 @@
 #include <QDBusConnectionInterface>
 #include <kconfiggroup.h>
 
-static const QString dbusServiceName = "org.kde.VisualNotifications";
-static const QString dbusInterfaceName = "org.kde.VisualNotifications";
-static const QString dbusPath = "/VisualNotifications";
+static const char dbusServiceName[] = "org.kde.VisualNotifications";
+static const char dbusInterfaceName[] = "org.kde.VisualNotifications";
+static const char dbusPath[] = "/VisualNotifications";
 
 NotifyByPopup::NotifyByPopup(QObject *parent) 
   : KNotifyPlugin(parent) , m_animationTimer(0), m_dbusServiceExists(false)
@@ -51,7 +53,7 @@ NotifyByPopup::NotifyByPopup(QObject *parent)
 	m_dbusServiceExists = interface && interface->isServiceRegistered(dbusServiceName);
 
 	if( m_dbusServiceExists )
-		kDebug(300) << "using" << dbusServiceName << "for popups";
+		slotServiceOwnerChanged(dbusServiceName, QString(), "_"); //connect signals
 
 	// to catch register/unregister events from service in runtime
 	connect(interface, SIGNAL(serviceOwnerChanged(const QString&, const QString&, const QString&)),
@@ -67,20 +69,21 @@ NotifyByPopup::~NotifyByPopup()
 
 void NotifyByPopup::notify( int id, KNotifyConfig * config )
 {
-	kDebug(300) << id;
+	kDebug(300) << id << "  active notifications:" << m_popups.keys() << m_idMap.keys();
+
+	if(m_popups.contains(id) || m_idMap.contains(id))
+	{
+		kDebug(300) << "the popup is already shown";
+		finish(id);
+		return;
+	}
 
 	// if Notifications DBus service exists on bus,
 	// it'll be used instead
 	if(m_dbusServiceExists)
 	{
-		sendNotificationDBus(id, 0, config);
-		return;
-	}
-
-	if(m_popups.contains(id))
-	{
-		//the popup is already shown
-		finish(id);
+		if(!sendNotificationDBus(id, 0, config))
+			finish(id); //an error ocurred.
 		return;
 	}
 
@@ -96,7 +99,8 @@ void NotifyByPopup::notify( int id, KNotifyConfig * config )
 	QString timeoutStr = config->readEntry( "Timeout" );
 	pop->setTimeout( !timeoutStr.isEmpty() ? timeoutStr.toInt() : 0 );
 
-	pop->show(QPoint(screen.left() + screen.width()/2  , m_nextPosition));
+	pop->adjustSize();
+	pop->show(QPoint(screen.left() + screen.width()/2 - pop->width()/2  , m_nextPosition));
 	m_nextPosition+=pop->height();
 }
 
@@ -165,20 +169,23 @@ void NotifyByPopup::slotLinkClicked( const QString &adr )
 
 void NotifyByPopup::close( int id )
 {
-	// if Notifications DBus service exists on bus,
-	// it'll be used instead
+	delete m_popups.take(id);
+
 	if( m_dbusServiceExists)
 	{
 		closeNotificationDBus(id);
-		return;
 	}
-
-	delete m_popups[id];
-	m_popups.remove(id);
 }
 
 void NotifyByPopup::update(int id, KNotifyConfig * config)
 {
+	if (m_popups.contains(id))
+	{
+		KPassivePopup *p=m_popups[id];
+		fillPopup(p, id, config);
+		return;
+	}
+
 	// if Notifications DBus service exists on bus,
 	// it'll be used instead
 	if( m_dbusServiceExists)
@@ -186,11 +193,6 @@ void NotifyByPopup::update(int id, KNotifyConfig * config)
 		sendNotificationDBus(id, id, config);
 		return;
 	}
-
-	if(!m_popups.contains(id))
-		return;
-	KPassivePopup *p=m_popups[id];
-	fillPopup(p, id, config);
 }
 
 void NotifyByPopup::fillPopup(KPassivePopup *pop,int id,KNotifyConfig * config)
@@ -201,16 +203,16 @@ void NotifyByPopup::fillPopup(KPassivePopup *pop,int id,KNotifyConfig * config)
 	KIconLoader iconLoader(iconName);
 	QPixmap appIcon = iconLoader.loadIcon( iconName, KIconLoader::Small );
 
-	KVBox *vb = pop->standardView( appCaption , config->pix.isNull() ? config->text : QString() , appIcon );
+	KVBox *vb = pop->standardView( appCaption , config->image.isNull() ? config->text : QString() , appIcon );
 	KVBox *vb2 = vb;
 
-	if(!config->pix.isNull())
+	if(!config->image.isNull())
 	{
-		const QPixmap &pix=config->pix;
+		QPixmap pix = QPixmap::fromImage(config->image.toImage());
 		KHBox *hb = new KHBox(vb);
 		hb->setSpacing(KDialog::spacingHint());
 		QLabel *pil=new QLabel(hb);
-		pil->setPixmap( config->pix );
+		pil->setPixmap( pix );
 		pil->setScaledContents(true);
 		if(pix.height() > 80 && pix.height() > pix.width() )
 		{
@@ -254,24 +256,20 @@ void NotifyByPopup::slotServiceOwnerChanged( const QString & serviceName,
 {
 	if(serviceName == dbusServiceName)
 	{
-		if(oldOwner.isEmpty())
+		kDebug(300) << serviceName << oldOwner << newOwner;
+		// tell KNotify that all existing notifications which it sent
+		// to DBus had been closed
+		foreach (int id, m_idMap.keys())
+			finished(id);
+		m_idMap.clear();
+
+		if(newOwner.isEmpty())
+		{
+			m_dbusServiceExists = false;
+		}
+		else if(oldOwner.isEmpty())
 		{
-			// delete existing popups if any
-			// NOTE: can't use qDeletaAll here, because it can happen that
-			// some passive popup auto-deletes itself and slotPopupDestroyed
-			// will get called *while* qDeleteAll will be iterating over QMap
-			// and that slot will remove corresponding key from QMap which will
-			// break qDeleteAll.
-			// This foreach takes this possibility into account:
-			foreach ( int id, m_popups.keys() )
-				delete m_popups.value(id,0);
-			m_popups.clear();
-
 			m_dbusServiceExists = true;
-			kDebug(300) << dbusServiceName << " was registered on bus, now using it to show popups";
-
-			// not forgetting to clear old assignments if any
-			m_idMap.clear();
 
 			// connect to action invocation signals
 			bool connected = QDBusConnection::sessionBus().connect(QString(), // from any service
@@ -281,7 +279,7 @@ void NotifyByPopup::slotServiceOwnerChanged( const QString & serviceName,
 					this,
 					SLOT(slotDBusNotificationActionInvoked(uint,const QString&)));
 			if (!connected) {
-				kDebug(300) << "warning: failed to connect to ActionInvoked dbus signal";
+				kWarning(300) << "warning: failed to connect to ActionInvoked dbus signal";
 			}
 
 			connected = QDBusConnection::sessionBus().connect(QString(), // from any service
@@ -291,20 +289,9 @@ void NotifyByPopup::slotServiceOwnerChanged( const QString & serviceName,
 					this,
 					SLOT(slotDBusNotificationClosed(uint,uint)));
 			if (!connected) {
-				kDebug(300) << "warning: failed to connect to NotificationClosed dbus signal";
+				kWarning(300) << "warning: failed to connect to NotificationClosed dbus signal";
 			}
 		}
-		if(newOwner.isEmpty())
-		{
-			m_dbusServiceExists = false;
-			// tell KNotify that all existing notifications which it sent
-			// to DBus had been closed
-			foreach (int id, m_idMap.keys()) {
-				finished(id);
-            }
-			m_idMap.clear();
-			kDebug(300) << dbusServiceName << " was unregistered from bus, using passive popups from now on";
-		}
 	}
 }
 
@@ -329,11 +316,13 @@ void NotifyByPopup::slotDBusNotificationClosed(uint dbus_id, uint reason)
 	Q_UNUSED(reason)
 	// find out knotify id
 	int id = m_idMap.key(dbus_id, 0);
+	kDebug(300) << dbus_id << "  -> " << id;
 	if (id == 0) {
 		kDebug(300) << "failed to find knotify id for dbus_id" << dbus_id;
 		return;
 	}
 	// tell KNotify that this notification has been closed
+	m_idMap.remove(id);
 	finished(id);
 }
 
@@ -344,8 +333,16 @@ void NotifyByPopup::getAppCaptionAndIconName(KNotifyConfig *config, QString *app
 	*iconName = globalgroup.readEntry("IconName", config->appname);
 }
 
-void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig* config)
+bool NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig* config)
 {
+	// figure out dbus id to replace if needed
+	uint dbus_replaces_id = 0;
+	if (replacesId != 0 ) {
+		dbus_replaces_id = m_idMap.value(replacesId, 0);
+		if (!dbus_replaces_id)
+			return false;  //the popup has been closed, there is nothing to replace.
+	}
+
 	QDBusMessage m = QDBusMessage::createMethodCall( dbusServiceName, dbusPath, dbusInterfaceName, "Notify" );
 
 	// NOTE readEntry here is not KConfigGroup::readEntry - this is a custom class
@@ -353,27 +350,8 @@ void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 	QString timeoutStr = config->readEntry( "Timeout" );
 	int timeout = !timeoutStr.isEmpty() ? timeoutStr.toInt() : 0;
 
-	// if timeout is still zero, try to set it to default knotify timeout
-	if (timeout == 0) {
-		// NOTE: this is a little hack. Currently there's no way to easily determine
-		// if KNotify will close notification after certain timeout or if it's persistent.
-		// The only thing that comes to mind is to check Persistent option in config and
-		// if it's absent, use default timeout that KNotify uses for non-persistent popups
-		bool persistent = (config->readEntry("Persistent") == "true" ||
-						   config->readEntry("Persistant") == "true");
-		if (!persistent) {
-			timeout = 6*1000;
-		}
-    }
-
 	QList<QVariant> args;
 
-	// figure out dbus id to replace if needed
-	uint dbus_replaces_id = 0;
-	if (replacesId != 0 ) {
-	    dbus_replaces_id = m_idMap.value(replacesId, 0);
-	}
-
 	QString appCaption, iconName;
 	getAppCaptionAndIconName(config, &appCaption, &iconName);
 
@@ -381,7 +359,7 @@ void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 	args.append( dbus_replaces_id ); // replaces_id
 	args.append( config->eventid ); // event_id
 	args.append( iconName ); // app_icon
-	args.append( QString()); // summary
+	args.append( config->title ); // summary
 	args.append( config->text ); // body
 	// galago spec defines action list to be list like
 	// (act_id1, action1, act_id2, action2, ...)
@@ -397,7 +375,13 @@ void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 	}
 
 	args.append( actionList ); // actions
-	args.append( QVariantMap() ); // hints - unused atm
+
+	QVariantMap map;
+	// let's see if we've got an image, and store the image in the hints map
+	if (!config->image.isNull())
+		map["image_data"] = config->image.data();
+
+	args.append( map ); // hints
 	args.append( timeout ); // expire timout
 
 	m.setArguments( args );
@@ -405,23 +389,40 @@ void NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 	if(replyMsg.type() == QDBusMessage::ReplyMessage) {
 		if (!replyMsg.arguments().isEmpty()) {
 			uint dbus_id = replyMsg.arguments().at(0).toUInt();
+			if (dbus_id == 0)
+			{
+				kDebug(300) << "error: dbus_id is null";
+				return false;
+			}
+			if (dbus_replaces_id && dbus_id == dbus_replaces_id) 
+				return true;
+#if 1
+			int oldId = m_idMap.key(dbus_id, 0);
+			if (oldId != 0) {
+				kWarning(300) << "Received twice the same id "<< dbus_id << "( previous notification: " << oldId << ")";
+				m_idMap.remove(oldId);
+				finish(oldId);
+			}
+#endif
 			m_idMap.insert(id, dbus_id);
-	 		kDebug() << "mapping knotify id to dbus id:"<< id << "=>" << dbus_id;
+			kDebug(300) << "mapping knotify id to dbus id:"<< id << "=>" << dbus_id;
+			return true;
 		} else {
-	 		kDebug() << "error: received reply with no arguments";
+			kDebug(300) << "error: received reply with no arguments";
 		}
 	} else if (replyMsg.type() == QDBusMessage::ErrorMessage) {
-		kDebug() << "error: failed to send dbus message";
+		kDebug(300) << "error: failed to send dbus message";
 	} else {
-		kDebug() << "unexpected reply type";
+		kDebug(300) << "unexpected reply type";
 	}
+	return false;
 }
 
 void NotifyByPopup::closeNotificationDBus(int id)
 {
-	uint dbus_id = m_idMap.value(id, 0);
+	uint dbus_id = m_idMap.take(id);
 	if (dbus_id == 0) {
-	    kDebug() << "not found dbus id to close";
+		kDebug(300) << "not found dbus id to close" << id;
 	    return;
 	}
 
@@ -433,8 +434,9 @@ void NotifyByPopup::closeNotificationDBus(int id)
 	bool queued = QDBusConnection::sessionBus().send(m);
 	if(!queued)
 	{
-		kDebug() << "warning: failed to queue dbus message";
+		kDebug(300) << "warning: failed to queue dbus message";
 	}
+	
 }
 
 #include "notifybypopup.moc"
